#!/usr/bin/perl -w
#
#  cofolga2ns.pl: A perl script for performing pairwise RNA sequence
#                 alignment by Cofolga2mo.
#
#  Usage:
#  cofolga2ns.pl [fasta file]
#

$os = "linux" ;
#$os = "win" ;

if ($os eq "linux") {
   # linux
   $exe = 'cofolga2ns ' ;
   $rm = 'rm -rf';
   $PWD = 'pwd' ;
   $rnafold = 'RNAfold -p' ;
   $SLASH = '\/' ;
} else { # win
   # windows ms-dos
   $exe = "..\cofolga2ns.exe  " ;
   $rm = "del /Q /F";
   $PWD = "cd" ;
   $rnafold = "RNAfold.exe -p";
   $SLASH = "\\" ;
}

# temporary directory/file names
$tempdir1 = $ARGV[0] ;
$tempdir1 =~ s/^.*\/// ;
$tempdir1 = $tempdir1.".cofolga2ns.wrkdir.$$" ;
$tempfile1 = "cofolga.temp1" ; # fasta 
$tempfile2 = "cofolga.temp2" ; # prob. mat.

$root = `$PWD` ;
$root =~ s/\n$// ;

open(IN,$ARGV[0]);

mkdir($tempdir1) ;
chdir($tempdir1) ;

# -f [fasta file]
# -pmf [BPP matrix file]
# -outdiv  -> output all individuals
# -pop [GA population size]
# -it [maximum GA iteration number]
# -outint [interavl for output, 0 does not output except for the final result]
# -unique  ->  not allow duplicated individuals in one GA population
# -r [initial random nubmer]
#
$options = "-f $tempfile1 -pmf $tempfile2 -outdiv -pop 50 -it 200 -outint 0 -unique -r 12345 ".join(' ', @ARGV[1..$#ARGV]) ;

# delete pre-existing RNAfold output files
system("$rm \*_dp.ps");

# the follwings are necessary for the correct output order of sequences.
my $i = 1 ; my @id = (); my $inp = '' ;
while(<IN>) {
   chomp;
   if (/^>/) {
      if ($i != 1) {
         $inp .= "\n" ;
      }
      $id[$i] = $_ ;
      $id[$i] =~ s/^>\s*// ;
      $inp .= sprintf("> %03d\n",$i) ;
      $i++ ;

   } else {
      # convert a letter othr than ACGUT to N
      $_ =~ tr/a-z/A-Z/ ;
      $_ =~ s/[^ACGTU]/N/g ;
      $inp .= $_ ;
   }
}
$inp .= "\n" ;
my $nsq = $i - 1 ;

&run_RNAfold($inp) ;

# set input
open(OUT,">$tempfile2") ;
for ($i = 0 ; $i < $nsq ; $i++) { 
   &write_pmat($i) ;
}
close(OUT);

&run_exe ($options) ; 

&postprocess() ;


sub postprocess() {
   chdir($root) ;
   system("$rm $tempdir1");
}


sub run_RNAfold {
   my (@ps) ;
   open(OUT,">$tempfile1") ;
   print OUT $_[0];
   close(OUT);
   
   `$rnafold < $tempfile1` ;
   
   @ps = glob("*_dp.ps") ;
   &set_pmat_and_fa(@ps) ;
   system("$rm *_dp.ps *_ss.ps") ; #  do not delete $tempfile1
}


sub set_pmat_and_fa {
# derived from pmat.pl
# input: @_ (@files)
# output: (@fasta, @pmat, @fastaline, $ntot, @pmatline) (global)
   my ($i, $j, $f, @pm_a, @pm_b, @pm_c, $sw, $sq, @a, $num) ;
   my @files = @_ ;

   @fasta = () ; @pmat = () ;

   for($i = 0 ; $i <= $#files ; $i++) {
      $f = $files[$i] ;

      @pm_a = () ;@pm_b = () ;@pm_c = () ;
      $sw = 0 ;
      open(IN,"$f") ;
      while(<IN>) {
         chomp;
         if(/^\/sequence/) {
            $sq = "" ;
            while(<IN>) {
               if (/^\) \} def/) {last;}
               chomp ; 
               $_ =~ s/.$// ;  # delete \
               $sq .= $_ ;
            }

         } elsif (/^showpage/ and $sw == 1) {
            last ;
         } elsif (/^\%data starts here/) {
            $sw = 1 ;
         } elsif ($sw == 1 and /ubox$/) {
            $_ =~ s/ .box$// ;
            @a = () ; @a = split ;
            push(@pm_a, $a[0]) ;
            push(@pm_b, $a[1]) ;
            push(@pm_c, $a[2]*$a[2]) ; # because, $a[2] = sqrt(pij).
                                       # see _dp.ps file.
         }
      }
      close(IN) ;
      
      $id = $i+1 ;
      $fasta[$i][0] = sprintf("> %03d\n", $id) ;
      $fasta[$i][1] = "$sq\n" ;

      $num = $#pm_a + 1 ;
      $pmat[$i][0] = sprintf("> %d %03d\n", $num, $id) ;
      for ($j = 0 ; $j <= $#pm_a ; $j++) {
         $pmat[$i][$j+1] = "$pm_a[$j] $pm_b[$j] $pm_c[$j]\n" ;
      }
      $pmatline[$i] = $num + 1 ;

   }
}


sub run_exe {
# Input: $opt, $inp(local), $tempfile1, $pmat_name, $fname
# Output: @output (local), stdout
   my ($opt) = @_ ;

   @output = `$exe $opt` ;

   print @output ;
}


sub write_pmat {
# Input: (@fasta, @pmat, @fastaline, @pmatline) (global)
#        $i, $fname, $pmat_name
# Output: $inp
   my ($i) = @_ ;

   for (my $k = 0 ; $k < $pmatline[$i] ; $k++) {
      print OUT  $pmat[$i][$k] ;
   }
}

